<?php

// This file is part of Photos
// Copyright (c) 2001-2004 Alex King
// see LICENSE.txt for more information

// ---------------------------------------------------------------------------
// functions

function query_splice($query, $type) {
	if (isset($query) && $query != "") {
		$query .= " ".$type." "; 
		return $query;
	}
}

function query_add_property_array($query, $property_name, $property_value, $and_or='AND', $equal_like='LIKE', $fuzzy='%') {
	if (isset($property_value) && $property_value != "") {
		if (is_array($property_value)) {
			$property_array = $property_value;
		}
		else {
			$property_array = preg_split("/[\s,]+/", $property_value);
		}
		$property_query = "";
		for ($i = 0; $i < count($property_array); $i++)
		{
			$property_query = query_splice($property_query, $and_or).$property_name." $equal_like '".$fuzzy.$property_array[$i].$fuzzy."'"; 
		}
		$property_query = "( ".$property_query." )";
		$query = query_splice($query, 'and').$property_query;
	}
	return $query;
}

function query_add_multi_property_array($query, $properties, $property_value, $and_or='AND', $equal_like='LIKE', $fuzzy='%') {
	if (isset($property_value) && $property_value != "") {
		if (is_array($property_value)) {
			$property_array = $property_value;
		}
		else {
			$property_array = preg_split("/[\s,]+/", $property_value);
		}

		$property_query = "";
		for ($i = 0; $i < count($property_array); $i++) {
			$property_query .= " ( ";
			for ($ii = 0; $ii < count($properties); $ii++) {
				$property_query .= " ( ".$properties[$ii]." ".$equal_like." '".$fuzzy.$property_array[$i].$fuzzy."' ) ";
				if ($ii + 1 != count($properties)) {
					$property_query .= " OR ";
				}
			}
			$property_query .= " ) ";
			if ($i + 1 != count($property_array)) {
				$property_query .= " AND ";
			}
		}
		$query = query_splice($query, 'and').$property_query;
	}
	return $query;
}

function query_add_property($query, $property_name, $property_value, $equal_like='LIKE', $fuzzy='%', $and_or='AND') {
	if (isset($property_value) && $property_value != "") {
		$query = query_splice($query, $and_or)."( ".$property_name." $equal_like '".$fuzzy.$property_value.$fuzzy."' )";
	}
	return $query;
}

function query_add_date_range_property($query, $property_name, $property_value_month, $property_value_day, $property_value_year, $greater_less) {
	if (!empty($property_value_month) || !empty($property_value_day) || !empty($property_value_year)) {
		$property_value_date = $property_value_year.$property_value_month.$property_value_day;
		$query = query_splice($query, 'AND')."( ".$property_name." ".$greater_less." ".$property_value_date." )";
	}
	return $query;
}

function print_contact_sheet($array, $array_type, $columns = 3, $max = 999) {
	global $lang;
	if ($max > count($array)) {
		$max = count($array);
	}
	if (count($array) == 0) {
		print("<p style=\"padding-left: 5px;\">".$lang->str("no_photos_to_display")."</p>");
	}
	else {
// check array_type, can handle list of IDs or URLs
		$cell_width = ceil(100 / $columns);
		$extra = $max % $columns;
		if ($array_type == "URLs") {
// write out table
			print("	<table width=\"100%\" cellpadding=\"7\" cellspacing=\"0\" border=\"0\">\n");
			for ($i = 0; $i < $max; $i++) {
				if ($i == 0 || (($i + $columns) % $columns == 0)) {
					print("		<tr valign=\"top\">\n");
				}

				$last_slash_pos = strrpos($array[$i], "/") + 1;
				$file_name = substr($array[$i],$last_slash_pos);
				
				$temp = "temp_thumb_".$i;
				$thumbnail = save_JPEG_file_from_JPEG($array[$i], 160, "temp/", $temp, 70);

				print("			<td align=\"center\" width=\"".$cell_width."%\"><img src=\"".$thumbnail."\" class=\"onePixelBorder\"><br>".substr($file_name, (strrpos($file_name, "/") + 1))."</td>\n");
				if (($i + 1) == $max && $extra > 0) {
					for ($o = 0; $o < ($columns - $extra); $o++) {
						print("			<td align=\"center\" width=\"".$cell_width."%\">&nbsp;</td>\n");
					}
				}
				if (($i + 1) % $columns == 0) {
					print("		</tr>\n");
				}
			}
			print("		</tr>\n");
			print("	</table>\n");
		} 
		elseif ($array_type == "id_list") {
// write out table
			print("	<table width=\"100%\" cellpadding=\"7\" cellspacing=\"0\" border=\"0\">\n");
			for ($i = 0; $i < $max; $i++) {
				if ($i == 0 || (($i + $columns) % $columns == 0)) {
					print("		<tr valign=\"top\">\n");
				}
				$temp_image = new image;
				$temp_image = $temp_image->retrieve($array[$i]);
				
				print("			<td align=\"center\" width=\"".$cell_width."%\">\n");
				print("				<a href=\"index.php?screen=view&id=".$temp_image->id."\"><img src=\"image.php?id=".$temp_image->id."\" class=\"onePixelBorder\"></a><br>\n");
				print("				<a href=\"index.php?screen=view&id=".$temp_image->id."\">".substr($temp_image->URL, (strrpos($temp_image->URL, "/") + 1))."</a>\n");
				print("			</td>\n");
				if (($i + 1) == $max && $extra > 0) {
					for ($o = 0; $o < ($columns - $extra); $o++) {
						print("			<td align=\"center\" width=\"".$cell_width."%\">&nbsp;</td>\n");
					}
				}
				if (($i + 1) % $columns == 0) {
					print("		</tr>\n");
				}
			}
			print("		</tr>\n");
			print("	</table>\n");
		}
		else {
			print("<p class=\"error\">Please specify \$array_type as: URLs or id_list.");
		}
	}
}

function trim_add_elipsis($string, $limit = 100) {
	if (strlen($string) > $limit) {
		$string = substr($string, 0, $limit)."...";
	}
	return $string;
}

function get_copy_entry_values($id = 0) { // if $id is set, retrieve that record - otherwise get the last image entered
	
	global $page, $database;
	
	$temp = mysql_query("SELECT * FROM $database->table_name WHERE needdetails != '1' ORDER BY dateentered DESC LIMIT 20")
		or die("error: ".mysql_error());
	
	$i = 0;
	$js = 'var quickEntryData = new Array(';
	$select = '<select name="copy_entry_list" onclick="setCopyEntryValueType(\'last\');">'."\n";
	while ($image = mysql_fetch_object($temp)) {
		if ($i > 0) {
			$js .= '                              ,';
		}
		$js .= 'Array("'.$image->ID.'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->title).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->description).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->keywords).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->people).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->event).'"'."\n";
		$js .= '                                    ,"'.$image->indoors.'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->location).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->state).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->country).'"'."\n";
		$js .= '                                    ,"'.get_JS_string($image->photographer).'"'."\n";
		$js .= '                                    )'."\n";
		$select .= '<option value="'.$i.'">(#'.$image->ID.') '.html(trim_add_elipsis($image->title, 50)).'</option>'."\n";
		$i++;
	}
	$js .= ');';
	$select .= '</select>'."\n";
	
	if ($id != 0) {
		$image = new image;
		$image->retrieve($id);
		
		$js .= "\n";
		$js .= 'var quickEntryDataID = new Array("'.$image->ID.'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->title).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->description).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->keywords).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->people).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->event).'"'."\n";
		$js .= '                                ,"'.$image->indoors.'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->location).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->state).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->country).'"'."\n";
		$js .= '                                ,"'.get_JS_string($image->photographer).'"'."\n";
		$js .= '                                );'."\n";
		
		$page->init_string .= " copyEntryData('retrieved', 0);";
	}

	$data = array($js
	             ,$select
	             );
	return $data;
}

function get_JS_string($string) {
	$string = str_replace('"', '\"', $string);
	$string = str_replace("\r\n", "\\n", $string);
	$string = str_replace("\r", "\\n", $string);
	$string = str_replace("\n", "\\n", $string);
	return $string;
}

function load_photos($path) {
	global $photo_URLs, $custom;
	if ($handle = opendir($path)) {
		if (substr($path, -1, 1) != "/") {
			$path = $path."/";
		}
		while (false !== ($file = readdir($handle))) {
		
			set_time_limit(99999);

			if ($file != "." && $file != ".." && 
			    ($custom->ignore_hidden_files == 0 || 
			     ($custom->ignore_hidden_files == 1 && substr($file, 0, 1) != "."))) { 
				if (is_file($path.$file) && strtolower(substr($file, -4, 4)) == ".jpg") {
					$photo_URLs[] = $path.$file;
				}
				elseif (is_dir($path.$file)) {
					$temp = $path.$file."/";
					load_photos($temp);				
				}
			}
		}
		closedir($handle);
		if (is_array($photo_URLs) && count($photo_URLs) > 0) {
			sort($photo_URLs, SORT_STRING);
		}
	}
	else {
		$photo_URLs = array();
	}
	return $photo_URLs;
}

function clean_up_temp_dir() {
	$path = "temp/";
	if ($handle = opendir($path)) {
		$temp = mktime() - 7200;
		while (false !== ($file = readdir($handle))) {
			if ($file != "." && $file != ".." && $file != "INFO.txt") { 
// only delete files older than 2 hours
				if (filemtime($path.$file) < $temp) {
					unlink($path.$file);
				}
			}
		}
	}
	closedir($handle);
}

function save_JPEG_file_from_JPEG($original, $size = 160, $path = "temp/", $file_name = "temp", $quality = 70) {
// read a JPEG and write it back out to another file, can change size
	if (strtolower(substr($file_name, -4)) == ".jpg") {
		$new_filename = $path.$file_name;
	}
	else {
		$new_filename = $path.$file_name.".jpg";
	}
	$original = @imagecreatefromJPEG($original);
	if (!$original) {
		return false;
	}
	else {
// resize image
		set_time_limit(99999);

		if (imagesx($original) > imagesy($original)) {		
			$new_w = $size;
			$img_scale_ratio = imagesx($original)/$new_w;
			$new_h = imagesy($original)/$img_scale_ratio;
			$new_h = floor($new_h);
		}
		else {
			$new_h = $size;
			$img_scale_ratio = imagesy($original)/$new_h;
			$new_w = imagesx($original)/$img_scale_ratio;
			$new_w = floor($new_w);
		}
// save resized image as file
		if (gd_version() >= 2) {
			$temp_img = imagecreatetruecolor($new_w, $new_h);
		}
		else {
			$temp_img = imagecreate($new_w, $new_h);
		}
		imagecopyresized($temp_img,$original,0,0,0,0,$new_w,$new_h,imagesx($original),imagesy($original));
		$temp_img = imageJPEG($temp_img, $new_filename, $quality);
		
		if ($temp_img != false) {
			return $new_filename;
		}
		else {
			return false;
		}
	}
}

function create_temp_thumbnail($URL, $size = 160, $quality = 70) {
	global $database;
	$filename = save_JPEG_file_from_JPEG($URL, 160, "temp/", get_unique_id());
	$thumbnail = addslashes(fread(fopen($filename, "r"), filesize($filename)));
	unlink($filename);
	$result = mysql_query("INSERT INTO ".$database->temp_table." (url, thumbnail) VALUES ('".$URL."', '".$thumbnail."')") or die("error: ".mysql_error());
	if (!$result) {
		return false;
	}
}

function get_temp_thumbnail($URL) {
	global $database;
	$result = mysql_query("SELECT thumbnail FROM ".$database->temp_table." where url='".$URL."'");
	return mysql_result($result, 0 ,"thumbnail");
}

function read_exif_data_quick($path) {
	$temp = "temp/read_exif_data_quick.temp";
	$in = fopen($path, "r");
	$out = fopen($temp,"w");
	fwrite($out, fread($in, 15000) );
	fclose($in);
	fclose($out);
	
	require_once('exifer/exif.php');
	
	$exif = read_exif_data_raw($temp, 0);
	return flatten_exif_array($exif);
}

function flatten_exif_array($exif_data, $exif_array = array()) {
	foreach ($exif_data as $k => $v) {
		if (is_array($v)) {
			$exif_array = flatten_exif_array($v, $exif_array);
		}
		else {
			$exif_array[$k] = $v;
		}
	}
	return $exif_array;
}

function get_unique_id() {
	$id = microtime();
	$id = str_replace(" ", "_", $id);
	$id = str_replace(".", "_", $id);
	return $id;
}

function is_in_database($property, $value) {
	global $database;
	if ($property == "folder") {
		$query = "SELECT COUNT(ID) FROM $database->table_name WHERE (URL LIKE '".$value."%')";
	}
	else {
		$query = "SELECT COUNT(ID) FROM $database->table_name WHERE (".$property." = '".$value."')";
	}
	$result = mysql_query($query);
	if ($result != false) {
		$result = mysql_result($result, 0);
		if ($result > 0) {
			return true;
		}
		else {
			return false;
		}
	}
	else {
		return false;
	}
}

function folder_in_database($value) {
	global $database;
	$query = "SELECT COUNT(ID) FROM $database->table_name WHERE (URL LIKE '".$value."%')";
	$result = mysql_result(mysql_query($query),0);
	return $result;
}

function get_preview_dimensions($original) {
	$original = @imagecreatefromJPEG($original);
	if (!$original) {
		return false;
	}
	else {
		set_time_limit(99999);
		
		$dimensions = array();
// default sizes for preview images		
		$sizes = array(800
		              ,640
		              ,320
		              );
		
		for ($i = 0; $i < count($sizes); $i++) {
			if (imagesx($original) > imagesy($original)) {		
				$new_w = $sizes[$i];
				$img_scale_ratio = imagesx($original)/$new_w;
				$new_h = imagesy($original)/$img_scale_ratio;
				$new_h = floor($new_h);
			}
			else {
				$new_h = $sizes[$i];
				$img_scale_ratio = imagesy($original)/$new_h;
				$new_w = imagesx($original)/$img_scale_ratio;
				$new_w = floor($new_w);
			}
			$dimensions[] = array($new_w
				                 ,$new_h
				                 );
		}
		return $dimensions;
	}
}

function html($string, $js = 0, $br = 0) {
	$string = htmlspecialchars($string);
	if ($br == 1) {
		$string = str_replace("\n", '<br />', $string);
	}
	if ($js == 1) {
		$string = strtr($string,array("\n"=>"\\n","\r"=>"","\t"=>"","\0"=>""));
	}
	return $string;
}	

function print_HTML($temp) {
	print(html($temp));
}	

function microtime_diff($a, $b) {
   list($a_dec, $a_sec) = explode(" ", $a);
   list($b_dec, $b_sec) = explode(" ", $b);
   return $b_sec - $a_sec + $b_dec - $a_dec;
}

function EXIF_fraction($temp) {
	$temp = explode("/", $temp);
	return round(($temp[0]/$temp[1]),3);
}

function get_friendly_timestamp($timestamp) {
	global $custom;
	
	$year = substr($timestamp, 0, 4);
	$month = substr($timestamp, 4, 2);
	$day = substr($timestamp, 6, 2);
	$hour = substr($timestamp, 8, 2);
	$min = substr($timestamp, 10, 2);
	$sec = substr($timestamp, 12, 2);
	
	$date = mktime($hour, $min, $sec, $month, $day, $year);
	
	$date = $date + (3600 * $custom->server_time_difference);
	
	return date("Y", $date)."-".date("m", $date)."-".date("d",$date)." ".date("H", $date).":".$min.":".$sec;
}

function get_dropdown($select_name, $options, $key, $style = "width: 100%;", $extra = "") {
	$temp = "<select name=\"".$select_name."\" class=\"onePixelBorder\" style=\"".$style."\" ".$extra.">\n";
	for ($i = 0; $i < count($options); $i++) {
		if ($key == $options[$i][0]) {
			$temp .="	<option value=\"".$options[$i][0]."\" selected>".$options[$i][1]."</option>\n";
		}
		else {
			$temp .= "	<option value=\"".$options[$i][0]."\">".$options[$i][1]."</option>\n";
		}
	}
	$temp .= "</select>\n";
	return $temp;
}

function get_friendly_binary($binary) {
	global $lang;
	if ($binary == 1) {
		return $lang->str("yes");
	}
	else {
		return $lang->str("no");
	}
}

function custom_date($year, $month, $day) {
	global $custom;
	$date = str_replace("y", $year, $custom->date_format);
	$date = str_replace("m", $month, $date);
	$date = str_replace("d", $day, $date);
	
	return $date;
}

function get_friendly_shutter($value) {
	if ($value == 0) {
		return false;
	}
	else {
		if ($value <= .05) {
			$value = "1/".(round((1/$value)/5)*5);
		}
		else if ($value > .05 && $value < 1) {
			$value = "1/".round((1/$value), 1);
		}
		else {
			$value = round($value, 1);
		}
		return $value;
	}
}

function get_friendly_fnumber($value) {
	$value = round($value, 1);
	if (!strstr($value, ".")) {
		$value = $value.".0";
	}
	return $value;
}

function get_finder($path) {
	global $custom, $lang;
	$folders = array();
	$files = array();
	$combined = array();
	$string = '';
	
	if ($handle = opendir($path)) {
		while (false !== ($file = readdir($handle))) {
			if ($file != "." && $file != "..") { 
				if (is_dir($path.'/'.$file)) {
					$folders[strtolower($file)] = $file;
				}
				else if (is_file($path.'/'.$file) && strtolower(substr($file, -4, 4)) == ".jpg" && ($custom->ignore_hidden_files == 0 || ($custom->ignore_hidden_files == 1 && substr($file, 0, 1) != "."))) {
					$files[$file] = $file;
				}
			}
		}
		closedir($handle);
	
		uksort($folders, "strnatcasecmp");
		uksort($files, "strnatcasecmp");
		
		foreach ($folders as $k => $v) {
			$combined[] = '<td width="1%"><nobr>(<a href="index.php?screen=load_folder&action=check_path&path='.urlencode($path.'/'.$v).'">'.$lang->str("add").'</a>)&nbsp;</nobr></td>'
			             .'<td width="1%"><a href="finder.php?path='.urlencode($path.'/'.$v).'" target="finderFrame"><img src="images/icon_folder.gif" align="absbottom" border="0" /></a></td>'
			             .'<td width="33%">&nbsp;<a href="finder.php?path='.urlencode($path.'/'.$v).'" target="finderFrame">'.$v.'</a></td>'."\n";
		}
		foreach ($files as $k => $v) {
			$combined[] = '<td width="1%"><nobr>(<a href="index.php?screen=entry&action=load_image&cleanURL='.urlencode($path.'/'.$v).'">'.$lang->str("add").'</a>)&nbsp;</nobr></td>'
			             .'<td width="1%"><img src="images/icon_file.gif" align="absbottom" border="0" /></td>'
			             .'<td width="33%">&nbsp;'.$v.'</td>'."\n";
		}
		$string .= '<table width="100%" cellpadding="0" cellspacing="0"><tr>'."\n";
		if (count($combined) > 0) {
			$col = ceil(count($combined)/3);
			for ($i = 0; $i < $col; $i++) {
				$string .= $combined[$i];
				if ($i + $col < count($combined)) {
					$string .= $combined[$i + $col];
				}
				else {
					$string .= '<td colspan="3" width="33%"></td>';
				}
				if ($i + (2 * $col) < count($combined)) {
					$string .= $combined[$i + (2 * $col)];
				}
				else {
					$string .= '<td colspan="3" width="33%"></td>';
				}
				if ($i + (2 * $col) <= count($combined)) {
					$string .= '</tr>'."\n".'<tr>';
				}
			}
		}
		else {
			$string .= '<td>'.$lang->str("no_folders_or_files").'</td>';
		}
		$string .= '</tr></table>'."\n";
	}
	return $string;
}

function append($string, $append, $separator = ' ') {
	if ($string != $append && !empty($append)) {
		if (!empty($string)) {
			$string .= $separator.$append;
		}
		else {
			$string = $append;
		}
	}
	return $string;
}

function addslashes_array($old) {
	$new = array();
	if (is_array($old)) {
		foreach($old as $k => $v) {
			if (is_array($v)) {
				$new[$k] = addslashes_array($v);
			}
			else {
				$new[$k] = addslashes($v);
			}
		}
		return $new;
	}
	else {
		return $old;
	}
}

// thanks Simon Cortesi
function gd_version () {
	static $gd_version_number = null;
	if ($gd_version_number === null) {
		// Use output buffering to get results from phpinfo()
		// without disturbing the page we're in.  Output
		// buffering is "stackable" so we don't even have to
		// worry about previous or encompassing buffering.
		ob_start ();
		phpinfo (8);
		$module_info = ob_get_contents ();
		ob_end_clean ();
		if (preg_match("/\bgd\s+version\b[^\d\n\r]+?([\d\.]+)/i", 
		$module_info, $matches)) {
			$gd_version_number = $matches[1];
		} else {
			$gd_version_number = 0;
		}
	}
	return $gd_version_number;
}
?>
